// the function called when we submit the sign up form
function onSubmitSignupForm( e )
{
	// stop the form from submitting and get the data entered (first name and email)
	// NOTE: getInputValues() simply takes an empty object and fills the values from
	// inputs with the same name
	e.preventDefault();
	var $form 	= e.target;
	var data 	= getInputValues( { "FNAME":null, "EMAIL":null }, $form );
	
	// disable the form for the minute
	toggleFormEnabled( $form, false );
	
	// open a new window that we'll be able to control and redirect in (otherwise, we'll
	// get stung with a trying to open a popup, as when we get the result, we'll no longer
	// be in the trusted click event)
	var win = window.open( '/signup.html', '_blank' );
	win.onload = function() {
		// send the details to signup.html so we can populate the hidden form on that page
		win.setDetails( data.FNAME, data.EMAIL );
	}
	
	// check if the email is already signed up
	// NOTE: result might be null if we couldn't reach the server
	checkMailChimpEmail( data, function( result ){
		
		// reset and re-enable the form
		$form.reset();
		toggleFormEnabled( $form, true );

		// set the result in the other window
		// NOTE: the window might not have loaded yet
		if( typeof win.setResult == 'undefined' )
		{
			var f 		= win.onload; // replace the one we already have
			win.onload 	= function(){
				if( f )
					f();
				
				// send the result to the signup.html page, which if it's successful will be a
				// JSON object in the form:
				// {
				//	'success':true,
				//	'email_address':'email@email.com'
				//	'status':'pending|subscribed'
				// }
				// if it's not successful, it'll be in the form
				// {
				//	'success':false,
				//	'message':'reason why'
				// }
				win.setResult( result );
			}
		}
		else
			win.setResult( result );
	});
}

// sends an email to our script to check if they've already signed up for our list or not
function checkMailChimpEmail( data, onFinish )
{
	// send an ajax request to our php page
	var request = new XMLHttpRequest();
	request.open( 'POST', '/php/checkMailChimpUser.php', true ); // NOTE: async request
	request.setRequestHeader( 'Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8' );
	
	// add our callbacks, passing in the function to call when we're finished
	request.onload = function( e1 ) {
		onMailChimpRequestLoad( e1, onFinish );
	}
	request.onerror = function( e2 ) {
		onMailChimpRequestFail( e2, onFinish );	
	}
	
	// get our data an url-encoded params
	var params = Object.keys( data ).map( function( key ) {
		return key.toLowerCase() + '=' + encodeURIComponent( data[key] );
	}).join( '&' );
	
	// send the request
	request.send( params );
}

// called when we get a response from our php file
function onMailChimpRequestLoad( e, onFinish )
{
	var request = e.target;
	if( request.status == 200 ) // 200 = success, anything else = failure
	{
		var data = null;
		try 
		{ 
			data = JSON.parse( request.responseText ); 
		}
		catch( err ) {} // we don't care about the error, as it'll be printed in the fail callback
		
		// if we have some data, and it worked, then they've already signed up to our
		// list and we can just redirect them (NOTE: they may be marked as pending, so
		// you can optionally show a message telling them to validate their email address)
		if( data != null && data.success )
		{
			onFinishMailChimpRequest( request, data, onFinish );
			return;
		}
	}
	
	// if we get here, then it means that the request failed, or there was a problem
	onMailChimpRequestFail( e, onFinish );
}

// called when there was a problem in the request to our php file
function onMailChimpRequestFail( e, onFinish )
{
	var request = e.target;
	var data	= null;
	try
	{
		data = JSON.parse( request.responseText );
	}
	catch( err ) { console.error( "Couldn't convert JSON error response", err ); }
	
	// print our error message if we have one
	if( data != null )
		console.error( data );
	
	// we're not finished
	onFinishMailChimpRequest( request, data, onFinish );
}

// we're finished with our request, so clean everything up and reset it
function onFinishMailChimpRequest( request, data, onFinish )
{
	// clear our request callbacks so it can be GC'd
	request.onload	= null;
	request.onerror	= null;
	
	// call our finish callback if we have one
	if( onFinish )
		onFinish( data );
}

// get the values from inputs from a form (recursive) - used to find the values entered.
// NOTE: we can make our lives easier by just giving our inputs unique IDs, but this allows
// us to have multiple forms on the same page
function getInputValues( searchObj, parent )
{
	var $children 	= parent.children;
	var len 		= $children.length;
	for( var i = 0; i < len; i++ )
	{
		// if this is a div, recurse
		var nodeName = $children[i].nodeName;
		if( nodeName === 'DIV' )
		{
			getInputValues( searchObj, $children[i] );
			continue;
		}
		
		// if it's not an input, do nothing
		if( nodeName !== 'INPUT' )
			continue;
		
		// NOTE: this assumes that the inputs have a NAME set, not an ID
		var name = $children[i].name;
		if( name in searchObj )
			searchObj[name] = $children[i].value;
	}
	return searchObj; // our object should now have its values filled
}

// enabled/disables the elements in a form
function toggleFormEnabled( parent, isEnabled )
{
	var $children 	= parent.children;
	var len			= $children.length;
	for( var i = 0; i < len; i++ )
	{
		var nodeName = $children[i].nodeName;
		if( nodeName == 'INPUT' || nodeName == 'BUTTON' )
			$children[i].disabled = !isEnabled;
		else if( nodeName == 'DIV' )
			toggleFormEnabled( $children[i], isEnabled );
	}
}