<?php
// the keys for our string replace
const KEY_DATA_CENTER 	= '{DC}';
const KEY_LIST_ID		= '{LIST_ID}';
const KEY_EMAIL_HASH	= '{EMAIL_HASH}';

// constants for our call
const DATA_CENTER 	= 'REPLACE_WITH_YOUR_DATA_CENTER';
const LIST_ID 		= 'REPLACE_WITH_YOUR_LIST_ID';
const API_KEY 		= 'REPLACE_WITH_YOUR_API_KEY';
const URL  			= 'http://{DC}.api.mailchimp.com/3.0/lists/{LIST_ID}/members/{EMAIL_HASH}?fields=email_address,status';

// extract the email passed along and make sure it's valid
$email = isset( $_POST['email'] ) ? $_POST['email'] : null;
if( $email == null || !is_string( $email ) || !filter_var( $email, FILTER_VALIDATE_EMAIL ) )
{
	http_response_code( 400 );
	echo json_encode( array(
		'success' => false,
		'message' => 'Bad data'
	) );
	return;
}

// create the url that we're going to call
$keys 	= array( KEY_DATA_CENTER, KEY_LIST_ID, KEY_EMAIL_HASH );
$values	= array( DATA_CENTER, LIST_ID, md5( $email ) );
$url	= str_replace( $keys, $values, URL );

// get our info
$result 	= null;
$httpstatus	= 400;
try
{
	$curl = curl_init();
	curl_setopt( $curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC );			// the type of auth to use
	curl_setopt( $curl, CURLOPT_USERPWD, 'anystring:' . API_KEY );	// our login details
	curl_setopt( $curl, CURLOPT_URL, $url );						// the url we're accessing
	curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );				// return the transfer as a string
	curl_setopt( $curl, CURLOPT_TIMEOUT, 5 ); 						// 5 second timeout
	$result 	= curl_exec( $curl );
	$httpstatus	= curl_getinfo( $curl, CURLINFO_HTTP_CODE );
	curl_close( $curl );
}
catch( Exception $e )
{
	http_response_code( 503 );
	echo json_encode( array(
		'success' => false,
		'message' => 'Couldn\'t access MailChimp'
	) );
	return;
}

// check our return status. If it's not 200, then it can mean a few things:
//	- User isn't on the list
//	- MailChimp is down
//	- Bad request (e.g. api key)
//	- We're being throttled
if( $httpstatus !== 200 )
{
	http_response_code( 404 );
	echo json_encode( array(
		'success' => false,
		'message' => 'User not found'
	) );
	return;
}

// add our success to the resulting json
$json = json_decode( $result );
if( $json )
{
	$json->success	= true;
	$result 		= json_encode( $json );
}

// just return the result, which will be the email address and their status (subscribed, pending, etc)
echo $result;
?>